Method: Libis::Tools::ConfigFile#initialize

Defined in:
lib/libis/tools/config_file.rb

#initialize(file_or_hash = nil, opt = {}) ⇒ ConfigFile

Create a new ConfigFile instance. The optional argument can either be a Hash or a String. The argument is passed to the #<< method after initialization.

Parameters:

  • file_or_hash (String, Hash) (defaults to: nil)

    optional String or Hash argument to initialize the data.



35
36
37
# File 'lib/libis/tools/config_file.rb', line 35

def initialize(file_or_hash = nil, opt = {})
  super _file_to_hash(file_or_hash), opt
end