Class: Libis::Tools::MetsFile::DnxSection
- Defined in:
- lib/libis/tools/mets_dnx.rb
Overview
Base class for a DNX section in the METS file. Each DnxSection derived class has a unique tag which will
become the id of the
Direct Known Subclasses
AccessRightsPolicy, Collection, CreatingApplication, EnvHardwareRegistry, EnvSoftwareRegistry, Environment, EnvironmentDependencies, EnvironmentHardware, EnvironmentSoftware, FileFixity, GeneralFileCharacteristics, GeneralIECharacteristics, GeneralRepCharacteristics, Inhibitors, ObjectCharacteristics, PreservationLevel, Relationship, RetentionPolicy, SignatureInformation, WebHarvesting
Instance Method Summary collapse
-
#tag ⇒ Object
Instance method to access the class tag for DNX.
Instance Method Details
#tag ⇒ Object
Instance method to access the class tag for DNX.
10 11 12 13 14 |
# File 'lib/libis/tools/mets_dnx.rb', line 10 def tag _tag = self.class.name.split('::').last _tag[0] = _tag[0].downcase _tag end |